/*
 * Copyright (C) 2013 by egnite GmbH
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*
 * A simple Nut/OS program to demonstrate how to create a text file on
 * memory card.
 */

/* Include some Nut/OS header files. */
#include <dev/debug.h>
#include <dev/at91_mci.h>
#include <sys/device.h>
#include <fs/phatfs.h>

/* Include some standard header files. */
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <errno.h>

/*
 * Initialize required devices.
 */
static void InitDevices(void)
{
    /* Initialize debug port for stdout. */
    NutRegisterDevice(&devDebug, 0, 0);
    freopen("dbgu", "w", stdout);

    /* Register block device. */
    if (NutRegisterDevice(&devAt91Mci0, 0, 0) == 0) {
        /* Initialize the file system. */
        NutRegisterDevice(&devPhat0, 0, 0);
    }
}

/*
 * Write a given number of demo lines to a file.
 */
static void WriteFile(const char *path, int lines)
{
    FILE *fp;
    int i;

    /* Open file. */
    fp = fopen(path, "w");
    if (fp == NULL) {
        printf("Error %d opening %s\n", errno, path);
        return;
    }
    /* Write the text lines. */
    for (i = 1; i <= lines; i++) {
        putchar('.');
        fprintf(fp, "This is line number %d\n", i);
    }
    /* Close file. */
    fclose(fp);
}

/*
 * The program starts here.
 */
int main(void)
{
    /* Volume ID */
    int volid;

    /* Initialize all devices. */
    InitDevices();

    /* Print banner on console. */
    puts("\n=*= Memory card writer =*=");

    /* Mount memory card partition. */
    volid = _open("MCI0:1/PHAT0", _O_RDWR | _O_BINARY);
    if (volid == -1) {
        printf("Error %d mounting card\n", errno);
    } else {
        /* Write a number of lines to a file. */
        WriteFile("PHAT0:/TEST.TXT", 10);
        /* Unmount partition. */
        _close(volid);
    }

    /* Never return from main(). */
    puts("\n=*= Done =*=");
    for (;;);
}
